"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = __importStar(require("react"));
const nars_common_1 = require("nars-common");
const RemoteComponent_1 = require("./RemoteComponent");
const Error_1 = require("./Error");
function getInputPropEncoder(config, component, propsIn) {
    return rpcInterface => {
        let encodedProps = {};
        for (const propKey in config) {
            const propDefinition = config[propKey];
            const prop = propsIn[propKey];
            switch (propDefinition.type) {
                case nars_common_1.PropType.Input:
                    if (!propsIn.hasOwnProperty(propKey) && !propDefinition.optional) {
                        throw Error_1.requiredPropMissing(propKey, component);
                    }
                    const result = propDefinition.encode(prop, rpcInterface);
                    if (nars_common_1.isSuccess(result)) {
                        encodedProps[propKey] = result.value;
                    }
                    else {
                        throw Error_1.badPropType(propKey, prop, component);
                    }
                    break;
            }
        }
        return encodedProps;
    };
}
function encodeLocalProps(config, component, propsIn) {
    let encodedProps = {};
    for (const propKey in config) {
        const propDefinition = config[propKey];
        const prop = propsIn[propKey];
        switch (propDefinition.type) {
            case nars_common_1.PropType.Local:
                const localPropKey = propKey;
                encodedProps[localPropKey] = prop;
                if (propDefinition.localPropType === nars_common_1.LocalPropType.Opaque &&
                    propDefinition.isRequired !== "optional" &&
                    typeof prop === "undefined") {
                    throw Error_1.localPropMissing(propKey, component);
                }
                break;
        }
    }
    return encodedProps;
}
function createRemoteComponentWithSocketLikeOrUrl(socketLikeOrUrl, config) {
    const components = {};
    const useSocketLikeOrUrl = () => {
        return typeof socketLikeOrUrl === "string"
            ? RemoteComponent_1.useWebSocket(socketLikeOrUrl, true)
            : () => socketLikeOrUrl;
    };
    for (const componentName in config) {
        const Component = props => {
            const webSocket = useSocketLikeOrUrl();
            const localProps = encodeLocalProps(config[componentName], componentName, props);
            const encoder = getInputPropEncoder(config[componentName], componentName, props);
            const { ErrorComponent, LoadingComponent } = props;
            return (React.createElement(RemoteComponent_1.RemoteComponent, { webSocket: webSocket, name: String(componentName), inputProps: encoder, localProps: localProps, renderLoading: LoadingComponent ? () => React.createElement(LoadingComponent, null) : undefined, renderError: ErrorComponent ? () => React.createElement(ErrorComponent, null) : undefined }));
        };
        Component.displayName = componentName;
        components[componentName] = Component;
    }
    return components;
}
function createRemoteComponentWithUrl(url, config) {
    return createRemoteComponentWithSocketLikeOrUrl(url, config);
}
exports.createRemoteComponentWithUrl = createRemoteComponentWithUrl;
function createRemoteComponentWithWebSocket(webSocket, config) {
    return createRemoteComponentWithSocketLikeOrUrl(webSocket, config);
}
exports.createRemoteComponentWithWebSocket = createRemoteComponentWithWebSocket;
function createRemoteComponent(config) {
    const components = {};
    for (const componentName in config) {
        const Component = props => {
            const { ErrorComponent, LoadingComponent, webSocket } = props;
            const localProps = encodeLocalProps(config[componentName], componentName, props);
            const encoder = getInputPropEncoder(config[componentName], componentName, props);
            return (React.createElement(RemoteComponent_1.RemoteComponent, { webSocket: webSocket, name: componentName, inputProps: encoder, localProps: localProps, renderLoading: LoadingComponent ? () => React.createElement(LoadingComponent, null) : undefined, renderError: ErrorComponent ? () => React.createElement(ErrorComponent, null) : undefined }));
        };
        Component.displayName = componentName;
        components[componentName] = Component;
    }
    return components;
}
exports.createRemoteComponent = createRemoteComponent;
